import bpy
from bpy.types import UIList, Panel, Menu
   
def draw_ui(layout, context, is_uv):
    obj = context.object
    if not context.selected_objects or obj is None or obj.type != 'MESH':
        layout.label(text='No valid object selected')
        return

    operation_box = None

    #Hide texture button
    toggle_text = 'Hide Texture' if obj.uv_checker_texture else 'Show Texture'
    toggle_icon = 'TEXTURE' if obj.uv_checker_texture else 'MESH_PLANE'

    texture_box = layout.box()
    texture_container = texture_box.row()
    texture_container.operator('visualuv.toggle_texture', text=toggle_text, icon=toggle_icon)
    texture_container.scale_y = 1.5

    layout.separator(factor=0.1)
    
    toggle_text = 'Enable Stretching' if obj.uv_operation != 'UV_STRETCHING' else 'Disable Stretching'
    toggle_icon = 'HIDE_OFF' if obj.uv_operation == 'UV_STRETCHING' else 'HIDE_ON'

    enable_box = layout.box()
    enable_container = enable_box.row()
    enable_container.operator('visualuv.toggle_stretching', text=toggle_text, icon=toggle_icon)
    enable_container.scale_y = 1.5
    if obj.uv_operation == 'UV_STRETCHING':
        operation_box = enable_box

    layout.separator(factor=0.1)

    # UV normals button
    toggle_text = 'Enable UV Normals' if obj.uv_operation != 'UV_NORMALS' else 'Disable UV Normals'
    toggle_icon = 'HIDE_OFF' if obj.uv_operation == 'UV_NORMALS' else 'HIDE_ON'

    enable_box = layout.box()
    enable_container = enable_box.row()
    enable_container.operator('visualuv.toggle_normals', text=toggle_text, icon=toggle_icon)
    enable_container.scale_y = 1.5
    if obj.uv_operation == 'UV_NORMALS':
        operation_box = enable_box

    layout.separator(factor=0.1)

    # UV overlapping button
    if obj.mode == 'EDIT' and not bpy.context.tool_settings.use_uv_select_sync:
        toggle_text = 'Enable UV Overlapping' if obj.uv_operation != 'UV_OVERLAP' else 'Disable UV Overlapping'
        toggle_icon = 'HIDE_OFF' if obj.uv_operation == 'UV_OVERLAP' else 'HIDE_ON'

        enable_box = layout.box()
        enable_container = enable_box.row()
        enable_container.operator('visualuv.toggle_overlap', text=toggle_text, icon=toggle_icon)
        enable_container.scale_y = 1.5
        if obj.uv_operation == 'UV_OVERLAP':
            operation_box = enable_box

        layout.separator(factor=0.1)

    # UV island coloring button
    toggle_text = 'Enable Island Coloring' if obj.uv_operation != 'UV_ISLANDS' else 'Disable Island Coloring'
    toggle_icon = 'HIDE_OFF' if obj.uv_operation == 'UV_ISLANDS' else 'HIDE_ON'

    enable_box = layout.box()
    enable_container = enable_box.row()
    enable_container.operator('visualuv.toggle_islands', text=toggle_text, icon=toggle_icon)
    enable_container.scale_y = 1.5
    if obj.uv_operation == 'UV_ISLANDS':
        operation_box = enable_box

    layout.separator(factor=0.1)

    if obj.uv_operation == 'NONE' and not obj.uv_checker_texture:
        return

    # texture controls
    if obj.uv_checker_texture:
        subbox = texture_box.box()
        subbox.template_ID(obj, 'uv_image', open='image.open')
        subbox.prop(obj, 'uv_texture_scale', text='Texture Scale', slider=True)
        if is_uv:
            subbox.prop(obj, 'uv_texture_alpha', text='Texture Alpha', slider=True)
            subbox.prop(obj, 'uv_fill_texture', text='Texture Fill')
    
    # current operation controls
    if operation_box:   
        if obj.uv_operation == 'UV_STRETCHING':
            subbox = operation_box.box()
            split = subbox.split(factor=0.45)
            split.label(text='Stretching')      
            split.prop(obj, 'uv_stretch_type', text='')

            split = subbox.split(factor=0.75)
            if obj.uv_stretch_type in ('ANGLES', 'AREA'):
                split.prop(obj, 'uv_max_division', text='Max Division', slider=True)
            elif obj.uv_stretch_type == 'EDGE_LENGTH':
                split.prop(obj, 'uv_max_difference', text='Max Difference', slider=True)
            split.operator('visualuv.update', text='', icon='FILE_REFRESH')

    # controls for enabling additional features
    enable_features_column = layout.column()
    main_box = enable_features_column.box()
    main_box.operator('visualuv.update', text='Refresh', icon='FILE_REFRESH')
    main_box.prop(obj, 'uv_alpha', text='Overlay Opacity', slider=True)
    main_box.prop(obj, 'uv_auto_update', text='Auto-Update')
    main_box.prop(obj, 'uv_show_3D', text='Render 3D Viewport')
    main_box.prop(obj, 'uv_show_2D', text='Render UV Editor')
    if not is_uv:
        main_box.prop(obj, 'uv_show_wire', text='Wireframe')
        main_box.prop(obj, 'uv_backface_culling', text='Backface Culling')
    
    # change color box
    if obj.uv_operation != 'NONE':
        color_box =enable_features_column.box()
        color_box.prop(obj, 'uv_enable_color_change', text='Color Change')
        if obj.uv_enable_color_change:
            color_box.prop(obj, 'uv_hue_shift', text='Hue Shift', slider=True)
            color_box.prop(obj, 'uv_saturation', text='Saturation', slider=True)
            color_box.prop(obj, 'uv_value', text='Value', slider=True)

    if is_uv:
        return

    # change location box
    location_box = enable_features_column.box()
    location_box.prop(obj, 'uv_enable_position_change', text='Position Change')
    if obj.uv_enable_position_change:
        location_box.prop(obj, 'uv_location_offset', text='Location Offset')

    # explosion view box
    explosion_box = enable_features_column.box()
    explosion_box.prop(obj, 'uv_enable_explosion_view', text='Explosion View')
    if obj.uv_enable_explosion_view:
        explosion_box.prop(obj, 'uv_explosion_offset', text='Explosion Offset', slider=True)
    

class VISUALUV_PT_uv_tool_menu3d(bpy.types.Panel):
    bl_label = "Visualise UV layer"
    bl_idname = "VISUALUV_PT_uv_tool_menu3d"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'VisualUV'

    def draw(self, context):
        layout = self.layout
        draw_ui(layout, context, False)

class VISUALUV_PT_uv_tool_menu2d(bpy.types.Panel):
    bl_label = "Visualise UV layer"
    bl_idname = "VISUALUV_PT_uv_tool_menu2d"
    bl_space_type = 'IMAGE_EDITOR'
    bl_region_type = 'UI'
    bl_category = 'VisualUV'

    def draw(self, context):
        layout = self.layout
        draw_ui(layout, context, True)
